function [this, job_id] = enqueue( this, argv, envs, varargin )

% [this, job_id] = enqueue( this, argv, envs, 'block', ['job_id', id] )
%
% argv              cell array. argv{1} contains a function name and rest
%                   elements contain its args.
% envs              cell array of .mat file names. These files are loaded
%                   before starting the job.
% 'block'           execution will block until the job can be queued.
% ['job_id', id]    the given id is used as the job id. Uniqueness is _not_
%                   checked, but the automatically generated ids are
%                   quaranteed to stay unique if used later.
%
% job_id == 0 indicates an error (the job is not inserted).


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% check arguments
if ~isarg('block'), error('Non-blocking behaviour not yet supported!'); end
[flg, job_id] = isarg('job_id'); if ~flg, job_id = this.largest_id + 1; end


if ~this.quiet, fprintf( 'Paramaster: trying to enqueue a new job..\n' ); end


% block until system load permits new jobs
if wait_until_sysload_ok( this ) ~= 0
  job_id = 0;
  return;
end

% enqueue the job
save( [this.commdir 'paramaster_temp.mat'], 'argv', 'envs' );
if system([ 'mv ' this.commdir 'paramaster_temp.mat ' ...
            this.commdir 'parajob_' num2str( job_id ) '.mat' ]) ~= 0
  delete([ this.commdir 'paramaster_temp.mat' ])
  job_id = 0;
  return;
end

% handle the job ids
this.largest_id = max( this.largest_id, job_id );


if ~this.quiet, fprintf( 'Paramaster: new job enqueued.\n' ); end

return;
